#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Windows.Forms;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Volume Price Analysis
    /// </summary>
    [Description("Volume Price Analysis")]
    public class VPA2 : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int dTL = 60; // Default setting for DTL
            private double wbf = 1.5; // Default setting for Wbf
            private double nbf = 0.7; // Default setting for Nbf
            private int vlp = 30; // Default setting for Vlp
        // User defined variables (add any user defined variables below)
			private DataSeries Vrg;	//Volume moving average
			private DataSeries Vp3;	// +3 Volume SD
			private DataSeries Vp2;	// +2 Volume SD
			private DataSeries Vp1;	// +1 Volume SD
			private DataSeries Vn1;	// -1 Volume SD
			private DataSeries Vn2;	// -2 Volume SD
			private DataSeries Vn3;	// -3 Volume SD
			private double VolSD = 0.00;	//SD of Volume
			private DataSeries arg;	//Wilders average of Range
			private BoolSeries wrb;	// wide range bar
			private BoolSeries nrb;	// narrow range bar
			private BoolSeries Vl;	// lower volume bars
			private BoolSeries Vh;	// higher volume bars
			private BoolSeries Vb;
			private BoolSeries upbar;
			private BoolSeries dnbar;
			private DataSeries x;
			private double x1 = 0.00;
			private BoolSeries ucls;
			private BoolSeries dcls;
			private BoolSeries mcls;
			private BoolSeries Vlcls;
			private BoolSeries Vhcls;
			private DataSeries j;	// SMA(C,5)
			private DataSeries TLL;	// LinearRegSlope of j - lon term
			private DataSeries Tlm;	// LinearRegSlope of j - medium term
			private DataSeries tls; // LinearRegSlope of j - short term
			private string textMsg1 = "";
			private string textMsg2 = "";
			//the following are VPA conditions/signals
			private BoolSeries utbar;	//upthrust bar
			private BoolSeries utcond1;	//upthrust condition1
			private BoolSeries utcond2; //upthrust condition2
			private BoolSeries utcond3; //upthrust condition3
			private BoolSeries trbar;	
			private BoolSeries Hutbar;
			private BoolSeries Hutcond;
			private BoolSeries tcbar;
			private BoolSeries Scond1;
			private BoolSeries Scond2;
			private BoolSeries scond;
			private BoolSeries stdn0;
			private BoolSeries stdn;
			private BoolSeries stdn1;
			private BoolSeries stdn2;
			private BoolSeries bycond1;
			private BoolSeries bycond;
			private BoolSeries stvol;
			private BoolSeries ndbar;
			private BoolSeries nsbar;
			private BoolSeries nbbar;
			private BoolSeries nbbar1;
			private BoolSeries lvtbar;
			private BoolSeries lvtbar1;
			private BoolSeries lvtbar2;
			private BoolSeries dbar;
			private BoolSeries eftup;
			private BoolSeries eftupf1;
			private BoolSeries eftdn;
			private string TitleMsg = "";
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			Vrg = new DataSeries(this);
			Vp3 = new DataSeries(this);
			Vp2 = new DataSeries(this);
			Vp1 = new DataSeries(this);
			Vn1 = new DataSeries(this);
			Vn2 = new DataSeries(this);
			Vn3 = new DataSeries(this);
			arg = new DataSeries(this);
			wrb = new BoolSeries(this);
			nrb = new BoolSeries(this);
			Vl = new BoolSeries(this);
			Vh = new BoolSeries(this);
			Vb = new BoolSeries(this);
			upbar = new BoolSeries(this);
			dnbar = new BoolSeries(this);
			x = new DataSeries(this);
			
			ucls = new BoolSeries(this);
			dcls = new BoolSeries(this);
			mcls = new BoolSeries(this);
			Vlcls = new BoolSeries(this);
			Vhcls = new BoolSeries(this);
			j = new DataSeries(this);
			TLL = new DataSeries(this);
			Tlm = new DataSeries(this);
			tls = new DataSeries(this);
			utbar = new BoolSeries(this);	
			utcond1 = new BoolSeries(this);
			utcond2 = new BoolSeries(this); 
			utcond3 = new BoolSeries(this); 
			trbar = new BoolSeries(this);	
			Hutbar = new BoolSeries(this);
			Hutcond = new BoolSeries(this);
			tcbar = new BoolSeries(this);
			Scond1 = new BoolSeries(this);
			Scond2 = new BoolSeries(this);
			scond = new BoolSeries(this);
			stdn0 = new BoolSeries(this);
			stdn = new BoolSeries(this);
			stdn1 = new BoolSeries(this);
			stdn2 = new BoolSeries(this);
			bycond1 = new BoolSeries(this);
			bycond = new BoolSeries(this);
			stvol = new BoolSeries(this);
			ndbar = new BoolSeries(this);
			nsbar = new BoolSeries(this);
			nbbar = new BoolSeries(this);
			nbbar1 = new BoolSeries(this);
			lvtbar = new BoolSeries(this);
			lvtbar1 = new BoolSeries(this);
			lvtbar2 = new BoolSeries(this);
			dbar = new BoolSeries(this);
			eftup = new BoolSeries(this);
			eftupf1 = new BoolSeries(this);
			eftdn = new BoolSeries(this);			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
			if(CurrentBar < 2) return;
			//calculate Volume moving average and it's standard deviation
			Vrg.Set( SMA( Volume, vlp)[0] );
			VolSD = StdDev(Vrg, vlp)[0];
			Vp3.Set( Vrg[0] + 3.0 * VolSD );
			Vp2.Set( Vrg[0] + 2.0 * VolSD );
			Vp1.Set( Vrg[0] + 1.0 * VolSD );
			Vn1.Set( Vrg[0] - 1.0 * VolSD );
			Vn2.Set( Vrg[0] - 2.0 * VolSD );
			Vn3.Set( Vrg[0] - 3.0 * VolSD );
			// calculate some Volume infomration
			Vl.Set( Volume[0] < Volume[1] && Volume[0] < Volume[2] );
			Vh.Set( Volume[0] > Volume[1] && Volume[1] > Volume[2] );
			Vb.Set( Volume[0] > Vrg[0] || Volume[0] > Volume[1] );
			//  Calculate Range information
			arg.Set( Wilder(Range(), 30 )[0] );
			wrb.Set( Range()[0] > ( wbf * arg[0]) ? true : false );	//wind range bar?
			nrb.Set( Range()[0] < ( nbf * arg[0]) ? true : false );	//narrow range bar?
			// calculate some price infomation
			upbar.Set( Close[0] > Close[1]);
			dnbar.Set( Close[0] < Close[1]);
			x.Set( Range()[0] / (Close[0] - Low[0]) );
			x1 =( Close[0] - Low[0] == 0.00 ? arg[0] : x[0] );
			ucls.Set( x1 < 2.0);
			dcls.Set( x1 > 2.0);
			mcls.Set( x1 < 2.2 && x1 > 1.8);
			Vlcls.Set( x1 > 4.0);
			Vhcls.Set( x1 < 1.35);
			j.Set( SMA(Close, 5)[0] );
			TLL.Set( LinRegSlopeSFX( j, false, 40, 0)[0] );
			Tlm.Set( LinRegSlopeSFX( j, false, 15, 0)[0] );
			tls.Set( LinRegSlopeSFX( j, false, 5, 0)[0] );
			// end of calculations
			
			// paint the price bar 
			if ( tls[0] > 0.00 && Tlm[0] > 0.00 && TLL[0] > 0.00)
				BarColor = Color.Green;
			else if ( tls[0] > 0.00 && Tlm[0] > 0.00 && TLL[0] < 0.00)
				BarColor = Color.DarkGreen;
			else if ( tls[0] > 0.00 && Tlm[0] < 0.00 && TLL[0] < 0.00)
				BarColor = Color.LimeGreen;
			else if ( tls[0]< 0.00 && Tlm[0] < 0.00 && TLL[0] < 0.00)
				BarColor = Color.Red;
			else if ( tls[0]< 0.00 && Tlm[0] > 0.00 && TLL[0] > 0.00)
				BarColor = Color.LimeGreen;
			else if ( tls[0]< 0.00 && Tlm[0] < 0.00 && TLL[0] > 0.00)
				BarColor = Color.Purple;
			else BarColor = Color.Brown;
			// display the  1st information bar
			textMsg1 = "Vol:";
			if( Volume[0] > Vp2[0] )
				textMsg1 = textMsg1 + " Very High";
			else if( Volume[0] > Vp1[0] )
				textMsg1 = textMsg1 + " High";
			else if( Volume[0] > Vrg[0] )
				textMsg1 = textMsg1 + " Above Average";
			else if( Volume[0] < Vrg[0] && Volume[0] > Vn1[0] )
				textMsg1 = textMsg1 + " Less Than Average";
			else if( Volume[0] < Vn1[0] )
				textMsg1 = textMsg1 + " Low";
			else textMsg1 = textMsg1 + " ";
			textMsg1 = textMsg1 + "\nSpread:";
			if( Range()[0] > (arg[0] * 2.0) )
				textMsg1 = textMsg1 + " Wide";
			else if( Range()[0] > arg[0] )
				textMsg1 = textMsg1 + " Above Average";
			else
				textMsg1 = textMsg1 + " Narrow";
			textMsg1 = textMsg1 + "\nClose:";
			if( Vhcls[0] )
				textMsg1 = textMsg1 + " Very High";
			else if( ucls[0] )
				textMsg1 = textMsg1 + " High";
			else if( mcls[0] )
				textMsg1 = textMsg1 + " Mid";
			else if( dcls[0] )
				textMsg1 = textMsg1 + " Down";
			else 
				textMsg1 = textMsg1 + "Very Low";
			
			//DrawTextFixed( "Msg1", textMsg1, TextPosition.TopLeft);
			//display the second information
			textMsg2 = "Trend:";
			if( tls[0] > 0 )
				textMsg2 = textMsg2 + "  Shrt Trm-Up";
			else
				textMsg2 = textMsg2 + "  Shrt Trm-Down";
			if( Tlm[0] > 0 ) 
				textMsg2 = textMsg2 + "\nMid Trm-Up";
			else
				textMsg2 = textMsg2 + "\nMid Trm-Down";	
			if( TLL[0] > 0 ) 
				textMsg2 = textMsg2 + "  Lng Trm-Up";
			else
				textMsg2 = textMsg2 + "  Lng Trm-Down";
			DrawTextFixed( "Msg2", textMsg1 + "\n" + textMsg2, TextPosition.TopLeft);

			//VSA signals
			utbar.Set( wrb[0] && dcls[0] && tls[0] > 0.0 );
			utcond1.Set( utbar[1] && dnbar[0] );
			utcond2.Set( utbar[1] && dnbar[0] && Volume[0] > Volume[1] );
			utcond3.Set( utbar[0] && Volume[0] > 2.0 * Vrg[0] );
			trbar.Set( Volume[1] > Vrg[0] && upbar[0] && wrb[1] && dnbar[0] && dcls[0] && wrb[0] && TLL[0] > 0.0 & High[0] > MAX(High, 10)[0] );
			Hutbar.Set( upbar[1] && Volume[0] > 1.5 * Vrg[0] && dnbar[0] && dcls[0] && !wrb[0] && !utbar[0] );
			Hutcond.Set( Hutbar[1] && dnbar[0] && dcls[0] && !utbar[0] );
			tcbar.Set( upbar[1] && High[0] == MAX( High, 5)[0] && dnbar[0] && (dcls[0] || mcls[0]) && Volume[0] > Vrg[0] && !wrb[0] && !Hutbar[0] );
			Scond1.Set( utcond1[0] || utcond2[0] || utcond3[0] );
			Scond2.Set( !Scond1[1] );
			scond.Set( Scond1[0] && Scond2[0] );
			stdn0.Set( TLL[0] > 0 && Volume[0] > Volume[1] && dnbar[1] && upbar[1] && (ucls[0] || mcls[0]) && tls[0] < 0.0 && Tlm[0] < 0.0 );
			stdn.Set( Volume[0] > Volume[1] && dnbar[1] && upbar[0] && (ucls[0] || mcls[0]) && tls[0] < 0.0 && Tlm[0] < 0.0 );
			stdn1.Set( TLL[0] < 0.0 && Volume[0] > Vrg[0] * 1.5 && dnbar[1] && upbar[0] && (ucls[0] || mcls[0]) &&tls[0] < 0.0 && Tlm[0] < 0.0);
			stdn2.Set( tls[0] < 0.0 && Volume[1] < Vrg[0] && upbar[0] && Vhcls[0] && Volume[0] > Vrg[0] );
			bycond1.Set( stdn[0] || stdn1[0] );
			bycond.Set( upbar[0] && bycond1[1] );
			stvol.Set( Low[0] == MIN(Low, 5)[0] && (ucls[0] || mcls[0] ) && Volume[0] > 1.5*Vrg[0] && TLL[0] < 0.0 );
			ndbar.Set( upbar[0] && nrb[0] && Vl[0] && dcls[0] );
			nsbar.Set( dnbar[0] && nrb[0] && Vl[0] && dcls[0] );
			nbbar.Set( Close[0] > Close[1] && Volume[0] < Volume[1] && nrb[0] && x1 < 2.0 );
			nbbar1.Set( (Close[0] > Close[1] && Volume[0] < Volume[1] && Volume[0] < Volume[2] && x1 < 1.1) ? true : false );
			lvtbar.Set( Vl[0] && Low[0] < Low[1] && ucls[0] );
			lvtbar1.Set( Volume[0] < Vrg[0] && Low[0] < Low[1] && ucls[0] && TLL[0] > 0.0 && Tlm[0] > 0.0 && wrb[0] );
			lvtbar2.Set( lvtbar[1] && upbar[0] && ucls[0] );
			dbar.Set( Volume[0] > 2.0 * Vrg[0] && dcls[0] && upbar[0] && tls[0] > 0.0 && Tlm[0] > 0.0 && !Scond1[0] && !utbar[0] );
			eftup.Set( High[0] > High[1] && Low[0] > Low[1] && Close[0] > Close[1] && Close[0] >= ((High[0] - Low[0])* 0.7 + Low[0]) && Range()[0] > arg[0] && Volume[0] > Volume[1] );
			eftupf1.Set( eftup[1] && (utbar[1] || utcond1[0] || utcond2[0] || utcond3[0]) );
			eftdn.Set( High[0] < High[1] && Low[0] < Low[1] && Close[0] < Close[1] && Close[0] <= ((High[0] - Low[0])* 0.25 + Low[0]) && Range()[0] > arg[0] && Volume[0] > Volume[1] );
			
			// plot the shapes
			//Note:  NT does not have a DrawCircle so I used DrawDot
			if( utbar[0] && !scond[0] ) DrawSquare("s1" + CurrentBar, true, 0, High[0] + 4  * TickSize, Color.Red);
			if( trbar[0] ) DrawDot("s2" + CurrentBar, true, 0, High[0] + 8 * TickSize, Color.Blue);
			if( scond[0] ) DrawTriangleDown("s3" + CurrentBar, true, 0, High[0] + 1 * TickSize ,Color.Red);
			if( stdn[0] ) DrawSquare("s4" + CurrentBar, true, 0, Low[0] + -8 * TickSize, Color.Lime);
			if( stdn1[0] ) DrawSquare("s5" + CurrentBar, true, 0, Low[0] + -8 * TickSize, Color.Lime);
			if( lvtbar1[0] ) DrawSquare("s6" + CurrentBar, true, 0, Low[0] + -8 * TickSize, Color.Lime);
			if( lvtbar2[0] ) DrawTriangleUp("s7" + CurrentBar, true, 0, Low[0] + -8 * TickSize, Color.Yellow);
			if( stvol[0] ) DrawDiamond( "s8" + CurrentBar, true, 0, Low[0] + -4 * TickSize, Color.Lime);
			if( bycond[0] ) DrawTriangleUp("s9" + CurrentBar, true, 0, Low[0] +  -8 * TickSize, Color.Lime);
			if( Hutbar[0] ) DrawSquare("s10" + CurrentBar, true, 0, High[0]  + 4 * TickSize, Color.Blue);
			if( Hutcond[0] ) DrawTriangleDown("s11" + CurrentBar, true, 0, High[0] + -8 * TickSize ,Color.Blue);
			if( tcbar[0] ) DrawTriangleDown("s12" + CurrentBar, true, 0, High[0] + -8 * TickSize ,Color.Yellow);
			if( stdn2[0] ) DrawTriangleUp("s13" + CurrentBar, true, 0, Low[0] +  -8 * TickSize, Color.Aqua);
			if( dbar[0] ) DrawSquare("s14" + CurrentBar, true, 0, High[0]  + 4 * TickSize, Color.Blue);
			if( lvtbar[0] ) DrawSquare("s15" + CurrentBar, true, 0, Low[0]  + -4 * TickSize, Color.DeepPink);
			if( ndbar[0] ) DrawSquare("s16" + CurrentBar, true, 0, High[0]  + 4 * TickSize, Color.Blue);
			if( nsbar[0] ) DrawDot("s17" + CurrentBar, true, 0, Low[0] + -8 * TickSize, Color.Lime);
			if( eftup[0] ) DrawDot("s18" + CurrentBar, true, 0, Median[0] , Color.Turquoise);
			if( eftdn[0] ) DrawDot("s19" + CurrentBar, true, 0, Median[0] , Color.Yellow);
			// commentary
			
			// title
			TitleMsg = " ";
			if( utbar[0] ) TitleMsg += "\nAn Upthrust Bar. A sign of weakness. ";
			if( utcond1[0] ) TitleMsg += "\nA downbar after an Upthrust. Confirm weakness. ";
			if( utcond2[0] && !utcond1[0] ) TitleMsg += "\nA High Volume downbar after an Upthrust. Confirm weakness. ";
			if( utcond3[0] ) TitleMsg += "\nThis upthrust at very High Voume, Confirms weakness. ";
			if( stdn1[0] ) TitleMsg += "\nStrength seen returning after a down trend. High volume adds to strength. ";
			if( stdn0[0] && !stdn[0] ) TitleMsg += "\nStrength seen returning after a down trend. ";
			if( stdn[0] && !stdn1[0] ) TitleMsg += "\nStrength seen returning after a long down trend. ";
			if( lvtbar[0] ) TitleMsg += "\nTest for supply. ";
			if( lvtbar2[0] ) TitleMsg += "\nAn Upbar closing near High after a Test confirms strength. ";
			if( stdn2[0] ) TitleMsg += "\nHigh volume upbar closing on the high indicates strength. ";
			if( tcbar[0] ) TitleMsg += "\nHigh volume Downbar after an upmove on high volume indicates weakness. ";
			if( ndbar[0] ) TitleMsg += "\nNo Demand. A sign of Weakness. ";
			if( nsbar[0] ) TitleMsg += "\nNo Supply. A sign of Strength. ";
			if( eftup[0] ) TitleMsg += "\nEffort to Rise. Bullish sign. ";
			if( eftupf1[0] ) TitleMsg += "\nEffort to Move up has failed. Bearish sign. ";
			if( stvol[0] ) TitleMsg += "\nStopping volume. Normally indicates end of bearishness is nearing. ";
			// following conditions appear in the "Commentary Section"
			if( Hutbar[0]) TitleMsg += "\nA pseudo Upthrust. This normally appears after an Up Bar with above average volume. ";
			if( Hutcond[0]) TitleMsg += "\nA downbar after a pseudo Upthrust Confirms weakness. ";
			if( dbar[0] ) TitleMsg += "\nA wide range, high volume bar in a up trend closing down is an indication the Distribution is in progress. ";
			DrawTextFixed( "titleMsg", TitleMsg, TextPosition.TopRight);
			
			

        }

        #region Properties

        [Description("Linear Regression period")]
        [Category("Parameters")]
        public int DTL
        {
            get { return dTL; }
            set { dTL = Math.Max(1, value); }
        }

        [Description("WRB factor")]
        [Category("Parameters")]
        public double Wbf
        {
            get { return wbf; }
            set { wbf = Math.Max(1, value); }
        }

        [Description("NRB factor")]
        [Category("Parameters")]
        public double Nbf
        {
            get { return nbf; }
            set { nbf = Math.Max(1, value); }
        }

        [Description("Volume lookback period")]
        [Category("Parameters")]
        public int Vlp
        {
            get { return vlp; }
            set { vlp = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VPA2[] cacheVPA2 = null;

        private static VPA2 checkVPA2 = new VPA2();

        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        public VPA2 VPA2(int dTL, double nbf, int vlp, double wbf)
        {
            return VPA2(Input, dTL, nbf, vlp, wbf);
        }

        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        public VPA2 VPA2(Data.IDataSeries input, int dTL, double nbf, int vlp, double wbf)
        {
            checkVPA2.DTL = dTL;
            dTL = checkVPA2.DTL;
            checkVPA2.Nbf = nbf;
            nbf = checkVPA2.Nbf;
            checkVPA2.Vlp = vlp;
            vlp = checkVPA2.Vlp;
            checkVPA2.Wbf = wbf;
            wbf = checkVPA2.Wbf;

            if (cacheVPA2 != null)
                for (int idx = 0; idx < cacheVPA2.Length; idx++)
                    if (cacheVPA2[idx].DTL == dTL && Math.Abs(cacheVPA2[idx].Nbf - nbf) <= double.Epsilon && cacheVPA2[idx].Vlp == vlp && Math.Abs(cacheVPA2[idx].Wbf - wbf) <= double.Epsilon && cacheVPA2[idx].EqualsInput(input))
                        return cacheVPA2[idx];

            VPA2 indicator = new VPA2();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.DTL = dTL;
            indicator.Nbf = nbf;
            indicator.Vlp = vlp;
            indicator.Wbf = wbf;
            indicator.SetUp();

            VPA2[] tmp = new VPA2[cacheVPA2 == null ? 1 : cacheVPA2.Length + 1];
            if (cacheVPA2 != null)
                cacheVPA2.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheVPA2 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VPA2 VPA2(int dTL, double nbf, int vlp, double wbf)
        {
            return _indicator.VPA2(Input, dTL, nbf, vlp, wbf);
        }

        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        public Indicator.VPA2 VPA2(Data.IDataSeries input, int dTL, double nbf, int vlp, double wbf)
        {
            return _indicator.VPA2(input, dTL, nbf, vlp, wbf);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VPA2 VPA2(int dTL, double nbf, int vlp, double wbf)
        {
            return _indicator.VPA2(Input, dTL, nbf, vlp, wbf);
        }

        /// <summary>
        /// Volume Price Analysis
        /// </summary>
        /// <returns></returns>
        public Indicator.VPA2 VPA2(Data.IDataSeries input, int dTL, double nbf, int vlp, double wbf)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VPA2(input, dTL, nbf, vlp, wbf);
        }

    }
}
#endregion
