// 
// Much code borrowed from "VolumeZones" indicator, Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// Modified to display rolling Market Profile by Ben L, sbgtrading@yahoo.com
//  The profile is calculated from the bars currently displayed on the chart.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections.Generic;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{	
	/// <summary>
	///
	/// </summary>
	[Description("")]
	public class VolProf : Indicator
	{
		#region Variables
		private double[] MP = new double [1000]; 

		private SolidBrush		histoBrush;
		private SolidBrush		POCBrush;
		private Color			POCcolor		= Color.Black;
		private Color			histoColor		= Color.Pink;
		private int				barSpacing		= 2;
		private CalculateValueAreaHisto_v2_MPtype profileName		= CalculateValueAreaHisto_v2_MPtype.VWTPO;
		private int				transparency	= 50;
		private double			pctOfWidth      = 0.50;
		private int				minutesformp    = 10080;
		private CalculateValueAreaHisto_v2_TextLoc pTextLoc = CalculateValueAreaHisto_v2_TextLoc.BottomRight;
		private double 			ChartHighPrice,ChartLowPrice,PricePtr,volumeMax;
		private double			MPHighPrice, MPLowPrice;
		private string FS;
		private int 			PriceDigits, firstMPbar, lastBar, lastChartBar, firstChartBar;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			ChartOnly				= true;
			Overlay					= true;
			CalculateOnBarClose		= true;
			PriceDigits = Math.Max(0,TickSize.ToString().Length-2);
			RecalculateColors();
			if(TickSize.ToString().Length<=2) FS="0";
			if(TickSize.ToString().Length==3) FS="0.0";
			if(TickSize.ToString().Length==4) FS="0.00";
			if(TickSize.ToString().Length==5) FS="0.000";
			if(TickSize.ToString().Length>=6) FS="0.0000";
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
		}

		private int GetYPos(double price, Rectangle bounds, double min, double max)
		{
			return (int) ((bounds.Y + bounds.Height) - 
				((price - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height);
		}
		private int GetXPos(int baridx, Rectangle bounds)
		{
			double PctFromRight = (lastChartBar - baridx);
			PctFromRight = PctFromRight / ((lastChartBar-firstChartBar)*1.0);
			int PixelFromRight = (int) Math.Round(bounds.Right - bounds.Width * PctFromRight);
			if(PixelFromRight > bounds.Right) PixelFromRight = bounds.Left;
			return PixelFromRight;
		}

		/// <summary>
		/// Called when the indicator is plotted.
		/// </summary>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			string POCdescription="";
			
			if(ChartControl == null) return;
			lastChartBar	= Math.Min(ChartControl.LastBarPainted, Bars.Count - 1);
			firstChartBar	= (lastChartBar - ChartControl.BarsPainted) + 1;
			DateTime firstMPbarTime = Bars.Get(lastChartBar).Time.AddMinutes(-minutesformp);
			firstMPbar = Bars.GetBar(firstMPbarTime)+1;
//			Print(ChartControl.Right.ToString());
			
			// Find highest and lowest price points
			MPHighPrice = 0.0;
			MPLowPrice  = double.MaxValue;
			int i;
			for (i = firstMPbar; i <= lastChartBar && i >= 0; i++)
			{
				MPHighPrice = Math.Max(MPHighPrice, Bars.Get(i).High);
				MPLowPrice  = Math.Min(MPLowPrice , Bars.Get(i).Low);
			}
			ChartHighPrice = 0.0;
			ChartLowPrice  = double.MaxValue;
			for (i = (lastChartBar - ChartControl.BarsPainted) + 1; i <= lastChartBar && i >= 0; i++)
			{
				ChartHighPrice = Math.Max(ChartHighPrice, Bars.Get(i).High);
				ChartLowPrice  = Math.Min(ChartLowPrice , Bars.Get(i).Low);
			}
//			DrawHorizontalLine("H",MPHighPrice,Color.Blue,DashStyle.Solid,1);
//			DrawHorizontalLine("L",MPLowPrice,Color.Yellow,DashStyle.Solid,1);
			for(i=0;i<MP.Length;i++) MP[i]=0.0;

			if((MPHighPrice-MPLowPrice)/TickSize+2 >= MP.Length) 
			{	MPHighPrice = (MP.Length-1)*TickSize+MPLowPrice;
				DrawText("ErrorText","Array out of bounds:  Cannot display histo data above "+MPHighPrice.ToString()+" price level.  HIT F5 to re-initialize",0,Close[0],Color.Red);
			}
			int TicksOnChart = (int)Math.Round((ChartHighPrice - ChartLowPrice) / TickSize,0)*3;

			if(profileName==CalculateValueAreaHisto_v2_MPtype.VOC) GenerateProfile_VOC();
			else if(profileName==CalculateValueAreaHisto_v2_MPtype.TPO) GenerateProfile_TPO();
			else GenerateProfile_VWTPO();

			// Pass 2: Paint the histo bars
			i=0;
			PricePtr = MPLowPrice;
			int leftedge = GetXPos(firstMPbar-1,bounds);
			if(leftedge<0) leftedge=10; //keeps all histo bars on the chart, nothing drops off the left edge of chart
			while(PricePtr<=MPHighPrice)
			{
				int yUpper = GetYPos(PricePtr+TickSize/2.0, bounds, min, max) + (TicksOnChart>=bounds.Height ? 0:barSpacing);
				int yLower = GetYPos(PricePtr-TickSize/2.0, bounds, min, max);

				double x=0.0;
				x = MP[(int) Math.Round((PricePtr-MPLowPrice)/TickSize,0) + 1];
				int barWidth   = (int) (((bounds.Right-leftedge) * pctOfWidth) * (x / volumeMax));

				if(x == volumeMax) {
					graphics.FillRectangle(POCBrush, new Rectangle(leftedge, yUpper, barWidth, Math.Abs(yUpper - yLower)));
					if(pTextLoc != CalculateValueAreaHisto_v2_TextLoc.None) {
						if(POCdescription.Length==0) POCdescription = "POC at: "+PricePtr.ToString(FS);
						else POCdescription = POCdescription + " & " + PricePtr.ToString(FS);
					}
				}
				else
					graphics.FillRectangle(histoBrush, new Rectangle(leftedge, yUpper, barWidth, Math.Abs(yUpper - yLower)));
				PricePtr = PricePtr + TickSize;
			}
			if(pTextLoc == CalculateValueAreaHisto_v2_TextLoc.BottomRight)
				DrawTextFixed("POClocs",POCdescription, TextPosition.BottomRight);
			else if(pTextLoc == CalculateValueAreaHisto_v2_TextLoc.BottomLeft)
				DrawTextFixed("POClocs",POCdescription, TextPosition.BottomLeft);
			else if(pTextLoc == CalculateValueAreaHisto_v2_TextLoc.TopRight)
				DrawTextFixed("POClocs",POCdescription, TextPosition.TopRight);
			else if(pTextLoc == CalculateValueAreaHisto_v2_TextLoc.TopLeft)
				DrawTextFixed("POClocs",POCdescription, TextPosition.TopLeft);
			else if(pTextLoc == CalculateValueAreaHisto_v2_TextLoc.Center)
				DrawTextFixed("POClocs",POCdescription, TextPosition.Center);
		}

		private void GenerateProfile_VOC()
		{	volumeMax = 0.0;
			for (int idx = firstMPbar; idx <= lastChartBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double PricePtr = Math.Round(bar.Close,PriceDigits);
				int i = (int) Math.Round((PricePtr-MPLowPrice)/TickSize,0) + 1;
				MP[i] = MP[i]+bar.Volume;
				volumeMax = Math.Max(volumeMax,MP[i]);
			}
		}
		private void GenerateProfile_TPO()
		{	volumeMax = 0.0;
			for (int idx = firstMPbar; idx <= lastChartBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double H=Math.Round(bar.High,PriceDigits);
				double L=Math.Round(bar.Low,PriceDigits);
				double PricePtr = L;
				while(PricePtr<=H)
				{	int i = (int) Math.Round((PricePtr-MPLowPrice)/TickSize,0) + 1;
					MP[i] = MP[i]+1.0;
					volumeMax = Math.Max(volumeMax,MP[i]);
					PricePtr = PricePtr+TickSize;
				}
			}
		}
		private void GenerateProfile_VWTPO()
		{	volumeMax = 0.0;
			for (int idx = firstMPbar; idx <= lastChartBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double H=Math.Round(bar.High,PriceDigits);
				double L=Math.Round(bar.Low,PriceDigits);
				double PricePtr = L;
				double V = bar.Volume/((H-L)/TickSize+1);
				while(PricePtr <= H)
				{	int i = (int) Math.Round((PricePtr-MPLowPrice)/TickSize,0) + 1;
					MP[i] = MP[i]+V;
					volumeMax = Math.Max(volumeMax,MP[i]);
					PricePtr = PricePtr+TickSize;
				}
			}
		}
			
		private void RecalculateColors() 
		{
			int	alpha	 = (int) (255.0 * ((100.0 - transparency) / 100.0));

			histoBrush   = new SolidBrush(Color.FromArgb(alpha, histoColor.R, histoColor.G, histoColor.B));
			POCBrush     = new SolidBrush(Color.FromArgb(alpha, POCcolor.R, POCcolor.G, POCcolor.B));
		}

		#region Properties

		[XmlIgnore()]
		[Description("Color of histogram bars.")]
		[Category("Colors")]
		[Gui.Design.DisplayNameAttribute("Histo color")]
		public Color HistoColor
		{
			get { return histoColor; }
			set { histoColor = value; RecalculateColors(); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string HistoColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(histoColor); }
			set { histoColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of POC level.")]
		[Category("Colors")]
		[Gui.Design.DisplayNameAttribute("POC color")]
		public Color poc_color
		{
			get { return POCcolor; }
			set { POCcolor = value; RecalculateColors(); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string POCColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(POCcolor); }
			set { POCcolor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("The percentage of transparency of the histo bars.")]
		[Category("Parameters")]
		public int Transparency
		{
			get { return transparency; }
			set 
			{ 
				transparency = Math.Max(0 , value); 
				transparency = Math.Min(90, value);
			}
		}
		/// <summary>
		/// </summary>
		[Description("Location of POC description text")]
		[Category("Parameters")]
		public CalculateValueAreaHisto_v2_TextLoc TextLoc
		{
			get { return pTextLoc; }
			set { pTextLoc = value; }
		}
		/// <summary>
		/// </summary>
		[Description("Minute size for each MP")]
		[Category("Parameters")]
		public int MinutesForMP
		{
			get { return minutesformp; }
			set { minutesformp = Math.Max(0 , value); }
		}
		/// <summary>
		/// </summary>
		[Description("Valid profile types: VOC (volume on close), TPO (time price opportunities) or VWTPO (volume weighted TPO).")]
		[Category("Parameters")]
		public CalculateValueAreaHisto_v2_MPtype ProfileName
		{
			get { return profileName; }
			set { profileName = value; }
		}
		/// <summary>
		/// </summary>
		[Description("The percentage screen width consumed by histo")]
		[Category("Parameters")]
		public double PctOfWidth
		{
			get { return pctOfWidth; }
			set 
			{ 
				pctOfWidth = Math.Max(0.10 , value); 
				pctOfWidth = Math.Min(1.0, value);
			}
		}
#endregion
	}

}
	public enum CalculateValueAreaHisto_v2_TextLoc
	{	None,
		TopRight,
		BottomRight,
		TopLeft,
		BottomLeft,
		Center
	}
	public enum CalculateValueAreaHisto_v2_MPtype
	{	TPO,
		VOC,
		VWTPO
	}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private VolProf[] cacheVolProf = null;

        private static VolProf checkVolProf = new VolProf();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public VolProf VolProf(int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            return VolProf(Input, minutesForMP, pctOfWidth, profileName, textLoc, transparency);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public VolProf VolProf(Data.IDataSeries input, int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            checkVolProf.MinutesForMP = minutesForMP;
            minutesForMP = checkVolProf.MinutesForMP;
            checkVolProf.PctOfWidth = pctOfWidth;
            pctOfWidth = checkVolProf.PctOfWidth;
            checkVolProf.ProfileName = profileName;
            profileName = checkVolProf.ProfileName;
            checkVolProf.TextLoc = textLoc;
            textLoc = checkVolProf.TextLoc;
            checkVolProf.Transparency = transparency;
            transparency = checkVolProf.Transparency;

            if (cacheVolProf != null)
                for (int idx = 0; idx < cacheVolProf.Length; idx++)
                    if (cacheVolProf[idx].MinutesForMP == minutesForMP && Math.Abs(cacheVolProf[idx].PctOfWidth - pctOfWidth) <= double.Epsilon && cacheVolProf[idx].ProfileName == profileName && cacheVolProf[idx].TextLoc == textLoc && cacheVolProf[idx].Transparency == transparency && cacheVolProf[idx].EqualsInput(input))
                        return cacheVolProf[idx];

            VolProf indicator = new VolProf();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MinutesForMP = minutesForMP;
            indicator.PctOfWidth = pctOfWidth;
            indicator.ProfileName = profileName;
            indicator.TextLoc = textLoc;
            indicator.Transparency = transparency;
            indicator.SetUp();

            VolProf[] tmp = new VolProf[cacheVolProf == null ? 1 : cacheVolProf.Length + 1];
            if (cacheVolProf != null)
                cacheVolProf.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheVolProf = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolProf VolProf(int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            return _indicator.VolProf(Input, minutesForMP, pctOfWidth, profileName, textLoc, transparency);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.VolProf VolProf(Data.IDataSeries input, int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            return _indicator.VolProf(input, minutesForMP, pctOfWidth, profileName, textLoc, transparency);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.VolProf VolProf(int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            return _indicator.VolProf(Input, minutesForMP, pctOfWidth, profileName, textLoc, transparency);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.VolProf VolProf(Data.IDataSeries input, int minutesForMP, double pctOfWidth, CalculateValueAreaHisto_v2_MPtype profileName, CalculateValueAreaHisto_v2_TextLoc textLoc, int transparency)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.VolProf(input, minutesForMP, pctOfWidth, profileName, textLoc, transparency);
        }

    }
}
#endregion
